lkml.org 
[lkml]   [2002]   [Jan]   [15]   [last100]   RSS Feed
Views: [more markup]  [less markup]  [headers]  [forward] 
 
Messages in this thread
Patch in this message
/
DateTue, 15 Jan 2002 11:47:21 +0200
FromDmitri Kassatkine <>
Subjectusb.c patch -> successfuly read usb descriptors on some USB device
Hi,

Sometimes USB subsystem cannot get USB descriptor.

Jan 15 10:44:21 selma kernel: hub.c: USB new device connect on bus1/1, 
assign
ed device number 25
Jan 15 10:44:21 selma kernel: usb.c: couldn't get all of config descriptors
Jan 15 10:44:21 selma kernel: usb.c: unable to get device 25 
configuration (e
rror=-110)


Here is patch how to fix it for 2.4.17. It works for 2.4.16 as well.
 It necassary to read descriptor at once, not first 8 byte first.

It works well now for NSM USB Bluetooth dongle.

br, DMitri

-- 
 Dmitri Kassatkine
 Nokia Research Center / Helsinki
 Mobile: +358 50 4836365
 E-Mail: dmitri.kassatkine@nokia.com

--- usb.c.orig	Wed Nov 21 19:59:11 2001
+++ usb.c	Tue Jan 15 11:38:35 2002
@@ -2041,7 +2041,7 @@
 		return -ENOMEM;
 	}
 
-	buffer = kmalloc(8, GFP_KERNEL);
+	buffer = kmalloc(255, GFP_KERNEL);
 	if (!buffer) {
 		err("unable to allocate memory for configuration descriptors");
 		return -ENOMEM;
@@ -2049,9 +2049,9 @@
 	desc = (struct usb_config_descriptor *)buffer;
 
 	for (cfgno = 0; cfgno < dev->descriptor.bNumConfigurations; cfgno++) {
 		/* We grab the first 8 bytes so we know how long the whole */
 		/*  configuration is */
-		result = usb_get_descriptor(dev, USB_DT_CONFIG, cfgno, buffer, 8);
+		result = usb_get_descriptor(dev, USB_DT_CONFIG, cfgno, buffer, 255);
 		if (result < 8) {
 			if (result < 0)
 				err("unable to get descriptor");
@@ -2072,19 +2072,24 @@
 			goto err;
 		}
 
-		/* Now that we know the length, get the whole thing */
-		result = usb_get_descriptor(dev, USB_DT_CONFIG, cfgno, bigbuffer, length);
-		if (result < 0) {
-			err("couldn't get all of config descriptors");
-			kfree(bigbuffer);
-			goto err;
-		}	
+		if (length > 255) {
+			/* Now that we know the length, get the whole thing */
+			result = usb_get_descriptor(dev, USB_DT_CONFIG, cfgno, bigbuffer, length);
+			if (result < 0) {
+				err("couldn't get all of config descriptors");
+				kfree(bigbuffer);
+				goto err;
+			}	
 	
-		if (result < length) {
-			err("config descriptor too short (expected %i, got %i)", length, result);
-			result = -EINVAL;
-			kfree(bigbuffer);
-			goto err;
+			if (result < length) {
+				err("config descriptor too short (expected %i, got %i)", length, result);
+				result = -EINVAL;
+				kfree(bigbuffer);
+				goto err;
+			}
+		}
+		else {
+			memcpy(bigbuffer, buffer, length);
 		}
 
 		dev->rawdescriptors[cfgno] = bigbuffer;
@@ -2049,9 +2049,9 @@
 
 	for (cfgno = 0; cfgno < dev->descriptor.bNumConfigurations; cfgno++) {
-		/* We grab the first 8 bytes so we know how long the whole */
+		/* We grab the first 255 bytes so we know how long the whole */
 		/*  configuration is */
\
 
 \ /
  Last update: 2005-03-22 12:22    [from the cache]
©2003-2008